import threading
import vlc
from .player import Player
from .helper import handle_inputs


def main():
    player = Player()
    input_thread = threading.Thread(target=handle_inputs, args=(player,))
    input_thread.start()
    while True:
        if player.get_state().value == vlc.State.Ended and player.get_queue_size() > 0:
            player.play_next()
        if threading.active_count() != 2:
            break


if __name__ == "__main__":
    main()



import requests
import bs4
import re


def grab_search_query(search_query):
    """
    Returns the most relevant URL with the corresponding search query.

    :param search_query: str
    :return: str
    """
    search_query = search_query.replace(" ", "+")
    base_url = "https://www.youtube.com"
    url = base_url + "/results?sp=EgIQAVAU&q=" + search_query
    session = requests.get(url=url)
    soup = bs4.BeautifulSoup(session.content, "html.parser")
    videos = soup.findAll('a', attrs={'class': 'yt-uix-tile-link'})

    regex = re.compile("^/watch\?v=.*$")
    filtered_videos = []

    for item in videos:
        if regex.search(item["href"]):
            filtered_videos.append(item["href"])

    if len(filtered_videos) == 0:
        return None

    most_relevant_url = base_url + filtered_videos[0]
    return most_relevant_url


def handle_inputs(player):
    """
    Prompts the user for input and handles options they select.

    :return: None
    """
    search_query = input("Please enter the song you are searching for: ")
    url = grab_search_query(search_query)

    if url is not None:
        player.add_song(url)
    else:
        print("Song '" + search_query + "' not found.")

    prompt_printed = False
    user_opt = ""
    while True:
        if not prompt_printed:
            user_opt = input("Enter control option (Type 'help' for list of available options): ")
            prompt_printed = True
        else:
            if user_opt == 'help':
                print("\n"
                      "add [song] - Adds [song] to the queue\n"
                      "clear - Clears the queue\n"
                      "pause - Pause the current song\n"
                      "resume - Resumes the current song\n"
                      "queue - Prints the current queue\n"
                      "skip - Plays the next song in queue\n"
                      "cur - Get the title of the current song\n"
                      "time - Get the time of the current song\n"
                      "prev - Get the title of the previous song\n"
                      "rewind - Restart the current song\n"
                      "remove [pos] - Removes the song in position [pos] from the q\n"
                      "exit - Exits the program\n")

            elif user_opt[0:3] == 'add' and len(user_opt) > 4:
                query = user_opt[4:]
                url = grab_search_query(query)

                if url is not None:
                    player.add_song(url)
                    print("Added '" + query + "' to queue.")
                else:
                    print("Song '" + search_query + "' not found.")

            elif user_opt == 'clear':
                player.clear_queue()
                print("Cleared the queue.")

            elif user_opt == 'time':
                if player.get_current_song() is None:
                    print("No song currently playing")
                else:
                    length = convert_ms(player.get_length())
                    current = convert_ms(player.get_time())
                    position = player.get_position()
                    title = player.get_current_song().get_title()
                    pound = "=" * int(40 * position)
                    dash = "-" * int((40 * (1 - position)))
                    progress = "[" + pound + dash + "] (" + str(int(position * 100)) + "%) " + current + " of " + length
                    text = title + ": " + progress
                    print(text)

            elif user_opt == 'pause':
                player.pause()
                print("Pausing '" + player.get_current_song().get_title() + "'.")

            elif user_opt == 'resume':
                player.resume()
                print("Resuming '" + player.get_current_song().get_title() + "'.")

            elif user_opt == "queue":  # not displaying queue properly
                output = ""
                queue = player.get_queue()
                for i in range(len(queue)):
                    output += "[" + str(i + 1) + "] - " + queue[i].get_title() + "\n"
                print(output)

            elif user_opt == 'skip':
                player.skip()
                print("Skipping '" + player.get_previous_song().get_title() + "'.")

            elif user_opt == 'cur':
                cur_song = player.get_current_song()
                if cur_song is not None:
                    print("'" + cur_song.get_title() + "'.")
                else:
                    print("No song is currently playing.")

            elif user_opt == 'prev':
                prev_song = player.get_previous_song()
                if prev_song is not None:
                    print("'" + prev_song.get_title() + "'.")
                else:
                    print("No song was previously playing.")

            elif user_opt == 'rewind':
                player.rewind()

            elif user_opt[0:6] == 'remove':
                pos_not_int = False
                try:
                    pos = int(user_opt[7:]) - 1
                except:
                    print("Illegal position entered.")
                    pos_not_int = True

                if not pos_not_int:
                    if pos >= player.get_queue_size() or pos < 0:
                        print("Illegal position entered.")
                    else:
                        removed_song = player.remove_song(pos)
                        print("Removed " + removed_song.get_title() + " from queue")

            elif user_opt == 'exit':
                break

            else:
                print("Option '" + user_opt + "' not supported.")

            prompt_printed = False


def convert_ms(ms):
    """
    Converts milliseconds into h:m:s

    :param ms: int
    :return: str
    """
    seconds = (ms / 1000) % 60
    seconds = int(seconds)
    if seconds < 10:
        seconds = "0" + str(seconds)
    else:
        seconds = str(seconds)

    minutes = (ms / (1000 * 60)) % 60
    minutes = int(minutes)
    if minutes < 10:
        minutes = "0" + str(minutes)
    else:
        minutes = str(minutes)

    hours = (ms / (1000 * 60 * 60)) % 24
    hours = int(hours)
    if hours < 10:
        hours = "0" + str(hours)
    else:
        hours = str(hours)
    return hours + ":" + minutes + ":" + seconds



import pafy


class Song:
    def __init__(self, url, vlc):
        """
        Instantiates a Song object with a url and vlc object.

        :param url: str
        :param vlc: str
        :return: None
        """
        self.__url = url
        self.__vlc = vlc
        self.__audio = pafy.new(url)
        self.__media = self.__vlc.media_new(self.get_audio_url())

    def get_title(self):
        """
        Returns the title of this Song.

        :return: str
        """
        return self.__audio.title

    def get_length(self):
        """
        Returns the length of this Song.

        :return: int
        """
        return self.__audio.length

    def get_audio_url(self):
        """
        Returns the audio URL of this Song.

        :return: str
        """
        return self.__audio.getbestaudio().url

    def get_media(self):
        """
        Returns the media of this Song.

        :return: Media
        """
        return self.__media



from .vlc import *
from .song import Song


class Player:
    def __init__(self):
        """
        Instantiates a Player object.

        :return: None
        """
        self.__queue = []
        self.__current_song = None
        self.__next_song = None
        self.__previous_song = None

        self.__vlc_instance = Instance()
        self.__media_player = self.__vlc_instance.media_player_new()

    def get_queue_size(self):
        """
        Returns the size of the player queue

        :return: int
        """
        return len(self.__queue)

    def get_queue(self):
        """
        Prints the queue

        :return: list
        """
        return self.__queue

    def add_song(self, url):
        """
        Adds a song to the queue, given the url for the song.

        :return: none
        """
        new_song = Song(url, self.__vlc_instance)
        self.__queue.append(new_song)

        if self.get_queue_size() == 1:
            self.__next_song = new_song

        if not self.__media_player.is_playing() and self.__current_song is None:
            self.play_next()

    def remove_song(self, pos):
        """
        Removes a song from the queue, given the position in the queue, and returns the item removed.

        :param pos: int
        :return: Song
        """
        if self.__queue[pos] == self.__next_song:
            if not pos == self.get_queue_size() - 1:
                self.__next_song = self.__queue[pos+1]
            else:
                self.__next_song = None

        return self.__queue.pop(pos)

    def get_current_song(self):
        """
        Returns the current song being played.

        :return: Song
        """
        return self.__current_song

    def get_next_song(self):
        """
        Returns the next song that will be played.

        :return: Song
        """
        return self.__next_song

    def get_previous_song(self):
        """
        Returns the previous song that was played.

        :return: Song
        """
        return self.__previous_song

    def pause(self):
        """
        Pauses the current song.

        :return: none
        """
        self.__media_player.set_pause(True)

    def resume(self):
        """
        Resumes the current song.

        :return: none
        """
        self.__media_player.set_pause(False)

    def __stop(self):
        """
        Stops the current song, moves to the next song in queue.

        :return: none
        """
        self.__media_player.stop()

        self.__previous_song = self.__current_song
        self.__current_song = None

    def rewind(self):
        state = self.__media_player.get_state()
        self.__media_player.set_pause(0)
        self.__media_player.set_time(0)
        if state == State.Paused:
            self.__media_player.set_pause(1)

    def skip(self):
        """
        Skips the current song, moves to the next song in queue.

        :return: none
        """
        self.__stop()

        if len(self.__queue) > 0:
            self.play_next()

    def play_next(self):
        """
        Plays the first song in the queue.

        :return: none
        """
        if self.get_queue_size() == 0:
            print("No songs in queue to play.")
        else:
            if self.__next_song is not None:
                self.__current_song = self.__next_song
            else:
                self.__current_song = self.__queue[0]

            if self.get_queue_size() == 1:
                self.__next_song = None
            else:
                self.__next_song = self.__queue[1]

            self.__queue.pop(0)
            self.__media_player.set_media(self.__current_song.get_media())
            self.__media_player.play()

    def clear_queue(self):
        """
        Clears the current queue.

        :return: none
        """
        self.__queue = []

    def get_state(self):
        """
        Returns the state of the MediaPlayer.

        :return: int
        """
        return self.__media_player.get_state()

    def get_length(self):
        """
        Returns the length (in ms) of the current song.

        :return: int
        """
        return self.__media_player.get_length()

    def get_time(self):
        """
        Returns the time (in ms) of the current song.

        :return: int
        """
        return self.__media_player.get_time()

    def get_position(self):
        """
        Returns the position (between 0.0 to 1.0) of the current song.

        :return: double
        """
        return self.__media_player.get_position()



from setuptools import setup

setup(
    name='py-mp',
    packages=['pymp'],
    author='EddyMaric, ewyuan',
    author_email='eddy.maric98@gmail.com, eric.yuan@mail.utoronto.ca',
    description='A simple command-line music player',
    long_description='py-mp, short for python music player, uses command-line to take in inputs from the user and makes a search query on Youtube. py-mp will pull the most relevant video and will stream the audio to the computer.',
    license='MIT',
    url='http://github.com/ewyuan/py-mp',
    entry_points={
        'console_scripts': [
            'py-mp = pymp.__main__:main'
        ]
    },
    version='1.1.1',
    install_requires=['python-vlc', 'requests', 'beautifulsoup4', 'pafy', 'youtube-dl'],
)
